from . import ziPython
import re 

def autoDetect(daq, exclude = []):
    """
    Returns a string with the first device that is not
    in the exclude list. Will throw a RuntimeError if no
    device was found.
    """
    if not isinstance(daq,ziPython.ziDAQServer):
        raise RuntimeError("First argument must be an instance of ziPython.ziDAQServer");
    nodes = daq.listNodes('/', 0);
    devices = filter(lambda node: re.match("dev*", node, re.IGNORECASE), nodes);
    if not isinstance(exclude,list):
        exclude = [exclude];
    exclude = list(x.lower() for x in exclude);
    devices = filter(lambda node: node.lower() not in exclude, devices);
    if len(devices) == 0 :               
        raise RuntimeError("No DUT found. Make sure that the USB cable (HF2) or ethernet cable (UHF)is connected to the host and the device is turned on.");
    # Found at least one device -> selection valid.
    # Select the first one
    device = devices[0].lower();
    print("autoDetect selected the device %s for the measurement." % device);
    return device;

def devices(daq):
    """
    Returns a list of devices.
    """
    if not isinstance(daq,ziPython.ziDAQServer):
        raise RuntimeError("First argument must be an instance of ziPython.ziDAQServer");
    nodes = daq.listNodes('/', 0);
    devices = filter(lambda node: re.match("dev*", node, re.IGNORECASE), nodes);
    devices = list(x.lower() for x in list(devices));
    return devices;

def autoConnect(defaultPort = None):
    """
    Try to connect to a Zurich Instruments server with an available
    device. First check if a device is found by connecting to a server on
    defaultPort, if so create and return a connection to that ziServer. If no
    devices are found on that server, check if a device is found on the
    secondary port, if so create and return a connection to that server. By
    default, a connection is returned to an HF2 server. A runtime error is
    thrown if no devices are found.
    """
    if defaultPort == None:
        defaultPort = 8005
        secondaryPort = 8004
    elif defaultPort == 8005:
        secondaryPort = 8004
    elif defaultPort == 8004:
        secondaryPort = 8005
    else:
        raise RuntimeError("autoConnect(): input argument defaultPort (%d) must be either 8004 or 8005." %defaultPort);
    try:
        daq = ziPython.ziDAQServer('localhost',defaultPort);
        devs = devices(daq);
    except RuntimeError:
        devs = [];
    if len(devs) > 0:
        # We have a server running and a device, we're done
        print "autoConnect connected to a server on port %d." %defaultPort
        return daq
    try:
        daq = ziPython.ziDAQServer('localhost',secondaryPort);
        devs = devices(daq);
    except RuntimeError:
        devs = [];
    if len(devs) > 0:
        # We have a server running and a device, we're done
        print "autoConnect connected to a server on port %d." %secondaryPort
        return daq
    # Error checking, keep it simple
    raise RuntimeError("autoConnect() failed to find a running server connected to a device. Please ensure that the correct Zurich Instruments server is running for your device and that your device is connected.");
